@@ -16,14 +16,16 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import sys
+
 from os.path import commonprefix
 
 from gi.repository import Gtk
 
-from pynicotine import slskmessages
 from pynicotine.config import config
 from pynicotine.gtkgui.widgets.accelerator import Accelerator
 from pynicotine.logfacility import log
+from pynicotine.slskmessages import UserStatus
 from pynicotine.utils import add_alias
 from pynicotine.utils import get_alias
 from pynicotine.utils import is_alias
@@ -52,13 +54,14 @@ class ChatEntry:
         Accelerator("<Shift>Tab", entry, self.on_tab_complete_accelerator, True)
         Accelerator("Tab", entry, self.on_tab_complete_accelerator)
 
-        # Emoji Picker
-        try:
-            self.entry.set_property("show-emoji-icon", True)
+        # Emoji Picker (disable on Windows and macOS for now until we render emoji properly there)
+        if sys.platform not in ("win32", "darwin"):
+            try:
+                self.entry.set_property("show-emoji-icon", True)
 
-        except TypeError:
-            # GTK version not supported
-            pass
+            except TypeError:
+                # GTK version not supported
+                pass
 
         # Spell Check
         if config.sections["ui"]["spellcheck"]:
@@ -74,7 +77,7 @@ class ChatEntry:
 
     def on_enter(self, *_args):
 
-        if not self.core.logged_in:
+        if self.core.user_status == UserStatus.OFFLINE:
             return
 
         text = self.entry.get_text()
@@ -193,13 +196,13 @@ class ChatEntry:
 
         elif cmd in ("/j", "/join"):
             if args:
-                self.core.queue.append(slskmessages.JoinRoom(args))
+                self.core.chatrooms.show_room(args)
 
         elif cmd in ("/l", "/leave", "/p", "/part"):
             if args:
-                self.core.queue.append(slskmessages.LeaveRoom(args))
+                self.core.chatrooms.remove_room(args)
             else:
-                self.core.queue.append(slskmessages.LeaveRoom(self.entity))
+                self.core.chatrooms.remove_room(self.entity)
 
         elif cmd in ("/ad", "/add", "/buddy"):
             if args:
