@@ -36,10 +36,19 @@ from pynicotine.utils import encode_path
 """ Global Style """
 
 
-SETTINGS_PORTAL = None
+GTK_SETTINGS = Gtk.Settings.get_default()
 
-if "gi.repository.Adw" not in sys.modules:
+if not hasattr(GTK_SETTINGS, "reset_property"):
+    SYSTEM_FONT = GTK_SETTINGS.get_property("gtk-font-name")
+    SYSTEM_ICON_THEME = GTK_SETTINGS.get_property("gtk-icon-theme-name")
+
+USE_LIBADWAITA = ("gi.repository.Adw" in sys.modules)
+USE_COLOR_SCHEME_PORTAL = (sys.platform not in ("win32", "darwin") and not USE_LIBADWAITA)
+
+
+if USE_COLOR_SCHEME_PORTAL:
     # GNOME 42+ system-wide dark mode for GTK without libadwaita
+    SETTINGS_PORTAL = None
 
     class ColorScheme:
         NO_PREFERENCE = 0
@@ -48,9 +57,6 @@ if "gi.repository.Adw" not in sys.modules:
 
     def read_color_scheme():
 
-        if SETTINGS_PORTAL is None:
-            return None
-
         try:
             result = SETTINGS_PORTAL.call_sync(
                 "Read", GLib.Variant("(ss)", ("org.freedesktop.appearance", "color-scheme")),
@@ -87,24 +93,19 @@ if "gi.repository.Adw" not in sys.modules:
     except Exception as portal_error:
         log.add_debug("Cannot start color scheme settings portal, falling back to GTK theme preference: %s",
                       portal_error)
-
-GTK_SETTINGS = Gtk.Settings.get_default()
-
-if not hasattr(GTK_SETTINGS, "reset_property"):
-    SYSTEM_FONT = GTK_SETTINGS.get_property("gtk-font-name")
-    SYSTEM_ICON_THEME = GTK_SETTINGS.get_property("gtk-icon-theme-name")
+        USE_COLOR_SCHEME_PORTAL = None
 
 
 def set_dark_mode(enabled):
 
-    if "gi.repository.Adw" in sys.modules:
+    if USE_LIBADWAITA:
         from gi.repository import Adw  # pylint:disable=no-name-in-module
 
         color_scheme = Adw.ColorScheme.FORCE_DARK if enabled else Adw.ColorScheme.DEFAULT
         Adw.StyleManager.get_default().set_color_scheme(color_scheme)
         return
 
-    if not enabled:
+    if USE_COLOR_SCHEME_PORTAL and not enabled:
         color_scheme = read_color_scheme()
 
         if color_scheme is not None:
@@ -271,13 +272,24 @@ def set_global_css():
         /* Remove unwanted spacing from buttons */
         border-spacing: 0;
     }
+
+    .generic-popover contents {
+        /* Remove unwanted spacing from popovers */
+        padding: 0;
+    }
     """
 
     global_css_provider = Gtk.CssProvider()
 
     if GTK_API_VERSION >= 4:
         css = css + css_gtk3_20 + css_gtk4
-        global_css_provider.load_from_data(css)
+
+        try:
+            global_css_provider.load_from_data(css)
+
+        except TypeError:
+            # https://gitlab.gnome.org/GNOME/pygobject/-/merge_requests/231
+            global_css_provider.load_from_data(css.decode("utf-8"), length=-1)
 
         Gtk.StyleContext.add_provider_for_display(  # pylint: disable=no-member
             Gdk.Display.get_default(), global_css_provider, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION
@@ -549,7 +561,12 @@ def set_widget_fg_bg_css(widget, bg_color=None, fg_color=None):
         context.add_provider(css_providers[widget], Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
         context.add_class(class_name)
 
-    css_providers[widget].load_from_data(css.encode('utf-8'))
+    try:
+        css_providers[widget].load_from_data(css.encode('utf-8'))
+
+    except TypeError:
+        # https://gitlab.gnome.org/GNOME/pygobject/-/merge_requests/231
+        css_providers[widget].load_from_data(css, length=-1)
 
 
 def set_widget_font(widget, font):
@@ -573,24 +590,25 @@ def update_tag_visuals(tag, color=None, font=None):
         tag.set_property("underline", Pango.Underline.SINGLE)
 
     # Hotspots
-    if color in ("useraway", "useronline", "useroffline"):
+    if color not in ("useraway", "useronline", "useroffline"):
+        return
 
-        usernamestyle = config_ui["usernamestyle"]
+    usernamestyle = config_ui["usernamestyle"]
 
-        if usernamestyle == "bold":
-            tag.set_property("weight", Pango.Weight.BOLD)
-        else:
-            tag.set_property("weight", Pango.Weight.NORMAL)
+    if usernamestyle == "bold":
+        tag.set_property("weight", Pango.Weight.BOLD)
+    else:
+        tag.set_property("weight", Pango.Weight.NORMAL)
 
-        if usernamestyle == "italic":
-            tag.set_property("style", Pango.Style.ITALIC)
-        else:
-            tag.set_property("style", Pango.Style.NORMAL)
+    if usernamestyle == "italic":
+        tag.set_property("style", Pango.Style.ITALIC)
+    else:
+        tag.set_property("style", Pango.Style.NORMAL)
 
-        if usernamestyle == "underline":
-            tag.set_property("underline", Pango.Underline.SINGLE)
-        else:
-            tag.set_property("underline", Pango.Underline.NONE)
+    if usernamestyle == "underline":
+        tag.set_property("underline", Pango.Underline.SINGLE)
+    else:
+        tag.set_property("underline", Pango.Underline.NONE)
 
 
 def update_widget_visuals(widget, list_font_target="listfont"):
