@@ -51,6 +51,10 @@ class BaseImplementation:
 
         self.create_menu()
 
+    def unload(self):
+        # Implemented in subclasses
+        pass
+
     def create_item(self, text=None, callback=None, check=False):
 
         item = {"id": self.menu_item_id, "sensitive": True, "visible": True}
@@ -552,14 +556,16 @@ class StatusNotifierImplementation(BaseImplementation):
             )
 
         except GLib.Error as error:
-            if self.tray_icon is not None:
-                self.tray_icon.unregister()
-
+            self.unload()
             raise ImplementationUnavailable("StatusNotifier implementation not available: %s" % error) from error
 
         self.update_menu()
         self.update_icon_theme()
 
+    def unload(self):
+        if self.tray_icon is not None:
+            self.tray_icon.unregister()
+
     @staticmethod
     def check_icon_path(icon_name, icon_path):
         """ Check if tray icons exist in the specified icon path """
@@ -663,6 +669,9 @@ class StatusIconImplementation(BaseImplementation):
 
         self.gtk_menu = self.build_gtk_menu()
 
+    def unload(self):
+        self.hide()
+
     def on_status_icon_popup(self, _status_icon, button, _activate_time):
 
         if button == 3:
@@ -761,7 +770,7 @@ class TrayIcon:
             "org.kde.StatusNotifierWatcher",
             Gio.BusNameWatcherFlags.NONE,
             self.load,
-            None
+            self._on_watcher_unload
         )
 
     def load(self, *_args):
@@ -799,6 +808,18 @@ class TrayIcon:
 
         self.hide()
 
+    def unload(self, *_args):
+
+        self.available = False
+
+        if self.implementation:
+            self.implementation.unload()
+            self.implementation = None
+
+    def _on_watcher_unload(self, *_args):
+        if isinstance(self.implementation, StatusNotifierImplementation):
+            self.unload()
+
     def update_window_visibility(self):
         if self.implementation:
             self.implementation.update_window_visibility()
