@@ -1,8 +1,8 @@
-# COPYRIGHT (C) 2020-2021 Nicotine+ Team
+# COPYRIGHT (C) 2020-2022 Nicotine+ Contributors
 # COPYRIGHT (C) 2016-2017 Michael Labouebe <gfarmerfr@free.fr>
-# COPYRIGHT (C) 2008-2011 Quinox <quinox@users.sf.net>
-# COPYRIGHT (C) 2008 Gallows <g4ll0ws@gmail.com>
-# COPYRIGHT (C) 2006-2009 Daelstorm <daelstorm@gmail.com>
+# COPYRIGHT (C) 2008-2011 quinox <quinox@users.sf.net>
+# COPYRIGHT (C) 2008 gallows <g4ll0ws@gmail.com>
+# COPYRIGHT (C) 2006-2009 daelstorm <daelstorm@gmail.com>
 #
 # GNU GENERAL PUBLIC LICENSE
 #    Version 3, 29 June 2007
@@ -20,9 +20,13 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import json
+import sys
+
 from pynicotine.logfacility import log
 from pynicotine.utils import execute_command
 from pynicotine.utils import http_request
+from pynicotine.utils import human_length
 
 
 class NowPlaying:
@@ -59,6 +63,9 @@ class NowPlaying:
 
         if get_player is None:
             player = self.config.sections["players"]["npplayer"]
+
+            if sys.platform in ("win32", "darwin") and player == "mpris":
+                player = "lastfm"
         else:
             player = get_player()
 
@@ -69,17 +76,17 @@ class NowPlaying:
 
         result = None
 
-        try:
-            if player == "lastfm":
-                result = self.lastfm(command)
-            elif player == "other":
-                result = self.other(command)
-            elif player == "mpris":
-                result = self.mpris(command)
+        if player == "lastfm":
+            result = self.lastfm(command)
 
-        except RuntimeError:
-            log.add_important_error(_("Could not execute now playing code. Are you sure you picked the right player?"))
-            return None
+        elif player == "listenbrainz":
+            result = self.listenbrainz(command)
+
+        elif player == "other":
+            result = self.other(command)
+
+        elif player == "mpris":
+            result = self.mpris(command)
 
         if not result:
             return None
@@ -113,7 +120,7 @@ class NowPlaying:
         title = title.replace("$p", "%(program)s")
 
         title = title % self.title
-        title = ' '.join((x for x in title.replace('\r', '\n').split('\n') if x))
+        title = ' '.join(x for x in title.replace('\r', '\n').split('\n') if x)
 
         if title:
             if callback:
@@ -124,30 +131,36 @@ class NowPlaying:
         return None
 
     def lastfm(self, user):
-        """ Function to get the last song played via lastfm api """
-
-        import json
+        """ Function to get the last song played via Last.fm API """
 
         try:
-            (user, apikey) = user.split(';')
+            user, apikey = user.split(';')
 
         except ValueError:
-            log.add_important_error(_("lastfm: Please provide both your lastfm username and API key"))
+            log.add_important_error(_("Last.fm: Please provide both your Last.fm username and API key"))
             return None
 
         try:
             response = http_request(
                 "https", "ws.audioscrobbler.com",
                 "/2.0/?method=user.getrecenttracks&user=" + user + "&api_key=" + apikey + "&limit=1&format=json",
-                headers={"User-Agent": "Nicotine+"})
+                headers={"User-Agent": self.config.application_name})
 
         except Exception as error:
-            log.add_important_error(_("lastfm: Could not connect to audioscrobbler: %(error)s"), {"error": error})
+            log.add_important_error(_("Last.fm: Could not connect to Audioscrobbler: %(error)s"), {"error": error})
             return None
 
         try:
             json_api = json.loads(response)
-            lastplayed = json_api["recenttracks"]["track"][0]
+            lastplayed = json_api["recenttracks"]["track"]
+
+            try:
+                # In most cases, a list containing a single track dictionary is sent
+                lastplayed = lastplayed[0]
+
+            except KeyError:
+                # On rare occasions, the track dictionary is not wrapped in a list
+                pass
 
             self.title["artist"] = lastplayed["artist"]["#text"]
             self.title["title"] = lastplayed["name"]
@@ -156,14 +169,14 @@ class NowPlaying:
                 _("Last played"), self.title["artist"], self.title["album"], self.title["title"])
 
         except Exception:
-            log.add_important_error(_("lastfm: Could not get recent track from audioscrobbler: %(error)s"),
-                                    {"error": str(response)})
+            log.add_important_error(_("Last.fm: Could not get recent track from Audioscrobbler: %(error)s"),
+                                    {"error": response})
             return None
 
         return True
 
     def mpris(self, player):
-        """ Function to get the currently playing song via dbus mpris v2 interface """
+        """ Function to get the currently playing song via DBus MPRIS v2 interface """
 
         # https://media.readthedocs.org/pdf/mpris2/latest/mpris2.pdf
 
@@ -176,14 +189,10 @@ class NowPlaying:
         dbus_property = 'org.freedesktop.DBus.Properties'
 
         if not player:
-
-            dbus_proxy = Gio.DBusProxy.new_sync(self.bus,
-                                                Gio.DBusProxyFlags.NONE,
-                                                None,
-                                                'org.freedesktop.DBus',
-                                                '/org/freedesktop/DBus',
-                                                'org.freedesktop.DBus',
-                                                None)
+            dbus_proxy = Gio.DBusProxy.new_sync(
+                self.bus, Gio.DBusProxyFlags.NONE, None,
+                'org.freedesktop.DBus', '/org/freedesktop/DBus', 'org.freedesktop.DBus', None
+            )
 
             names = dbus_proxy.ListNames()
             players = []
@@ -193,7 +202,7 @@ class NowPlaying:
                     players.append(name[len(dbus_mpris_service):])
 
             if not players:
-                log.add_important_error(_("MPRIS: Could not find a suitable MPRIS player."))
+                log.add_important_error(_("MPRIS: Could not find a suitable MPRIS player"))
                 return None
 
             player = players[0]
@@ -201,28 +210,25 @@ class NowPlaying:
                 log.add(_("Found multiple MPRIS players: %(players)s. Using: %(player)s"),
                         {'players': players, 'player': player})
             else:
-                log.add(_("Auto-detected MPRIS player: %s."), player)
+                log.add(_("Auto-detected MPRIS player: %s"), player)
 
         try:
-            dbus_proxy = Gio.DBusProxy.new_sync(self.bus,
-                                                Gio.DBusProxyFlags.NONE,
-                                                None,
-                                                dbus_mpris_service + player,
-                                                dbus_mpris_path,
-                                                dbus_property,
-                                                None)
+            dbus_proxy = Gio.DBusProxy.new_sync(
+                self.bus, Gio.DBusProxyFlags.NONE, None,
+                dbus_mpris_service + player, dbus_mpris_path, dbus_property, None
+            )
 
             metadata = dbus_proxy.Get('(ss)', dbus_mpris_player_service, 'Metadata')
 
-        except Exception as exception:
+        except Exception as error:
             log.add_important_error(_("MPRIS: Something went wrong while querying %(player)s: %(exception)s"),
-                                    {'player': player, 'exception': exception})
+                                    {'player': player, 'exception': error})
             return None
 
         self.title['program'] = player
         list_mapping = [('xesam:artist', 'artist')]
 
-        for (source, dest) in list_mapping:
+        for source, dest in list_mapping:
             try:
                 self.title[dest] = '+'.join(metadata[source])
             except KeyError:
@@ -238,7 +244,7 @@ class NowPlaying:
             ('xesak:trackNumber', 'track')
         ]
 
-        for (source, dest) in mapping:
+        for source, dest in mapping:
             try:
                 self.title[dest] = str(metadata[source])
             except KeyError:
@@ -246,7 +252,7 @@ class NowPlaying:
 
         # The length is in microseconds, and be represented as a signed 64-bit integer.
         try:
-            self.title['length'] = self.get_length_time(metadata['mpris:length'] // 1000000)
+            self.title['length'] = human_length(metadata['mpris:length'] // 1000000)
         except KeyError:
             self.title['length'] = '?'
 
@@ -258,19 +264,43 @@ class NowPlaying:
 
         return True
 
-    @staticmethod
-    def get_length_time(seconds):
-        """ Function used to normalize tracks duration """
+    def listenbrainz(self, username):
+        """ Function to get the currently playing song via ListenBrainz API """
 
-        minutes, seconds = divmod(seconds, 60)
-        hours, minutes = divmod(minutes, 60)
+        if not username:
+            log.add_important_error(_("ListenBrainz: Please provide your ListenBrainz username"))
+            return None
 
-        if hours > 0:
-            ret = '{}:{:02}:{:02}'.format(hours, minutes, seconds)
-        else:
-            ret = '{}:{:02}'.format(minutes, seconds)
+        try:
+            response = http_request('https', 'api.listenbrainz.org',
+                                    '/1/user/{}/playing-now'.format(username),
+                                    headers={'User-Agent': self.config.application_name})
 
-        return ret
+        except Exception as error:
+            log.add_important_error(_("ListenBrainz: Could not connect to ListenBrainz: %(error)s"), {'error': error})
+            return None
+
+        try:
+            json_api = json.loads(response)['payload']
+
+            if not json_api['playing_now']:
+                log.add_important_error(_("ListenBrainz: You don\'t seem to be listening to anything right now"))
+                return None
+
+            track = json_api['listens'][0]['track_metadata']
+
+            self.title['artist'] = track['artist_name']
+            self.title['title'] = track['track_name']
+            self.title['album'] = track['release_name']
+            self.title['nowplaying'] = '%s: %s - %s - %s' % (
+                _('Playing now'), self.title['artist'], self.title['album'], self.title['title'])
+
+            return True
+
+        except Exception:
+            log.add_important_error(_("ListenBrainz: Could not get current track from ListenBrainz: %(error)s"),
+                                    {'error': str(response)})
+        return None
 
     def other(self, command):
 
@@ -281,6 +311,7 @@ class NowPlaying:
             output = execute_command(command, returnoutput=True)
             self.title["nowplaying"] = output
             return True
+
         except Exception as error:
             log.add_important_error(_("Executing '%(command)s' failed: %(error)s"),
                                     {"command": command, "error": error})
