@@ -110,6 +110,15 @@ class InternalMessage:
     msgtype = MessageType.INTERNAL
 
 
+class LogMessage(InternalMessage):
+
+    def __init__(self, callback=None, timestamp_format=None, msg=None, level=None):
+        self.callback = callback
+        self.timestamp_format = timestamp_format
+        self.msg = msg
+        self.level = level
+
+
 class CloseConnection(InternalMessage):
 
     def __init__(self, sock=None):
@@ -2353,6 +2362,7 @@ class SharedFileList(PeerMessage):
     def _parse_result_list(self, message, pos=0):
         pos, ndir = self.unpack_uint32(message, pos)
 
+        ext = None
         shares = []
         for _ in range(ndir):
             pos, directory = self.unpack_string(message, pos)
@@ -2365,7 +2375,7 @@ class SharedFileList(PeerMessage):
                 pos, code = self.unpack_uint8(message, pos)
                 pos, name = self.unpack_string(message, pos)
                 pos, size = self.parse_file_size(message, pos)
-                pos, ext = self.unpack_string(message, pos)
+                pos, _ext = self.unpack_string(message, pos)  # Obsolete, ignore
                 pos, numattr = self.unpack_uint32(message, pos)
 
                 attrs = {}
@@ -2373,7 +2383,7 @@ class SharedFileList(PeerMessage):
                 for _ in range(numattr):
                     pos, attrnum = self.unpack_uint32(message, pos)
                     pos, attr = self.unpack_uint32(message, pos)
-                    attrs[str(attrnum)] = attr
+                    attrs[attrnum] = attr
 
                 files.append((code, name, size, ext, attrs))
 
@@ -2492,12 +2502,13 @@ class FileSearchResult(PeerMessage):
     def _parse_result_list(self, message, pos):
         pos, nfiles = self.unpack_uint32(message, pos)
 
+        ext = None
         results = []
         for _ in range(nfiles):
             pos, code = self.unpack_uint8(message, pos)
             pos, name = self.unpack_string(message, pos)
             pos, size = self.parse_file_size(message, pos)
-            pos, ext = self.unpack_string(message, pos)
+            pos, _ext = self.unpack_string(message, pos)  # Obsolete, ignore
             pos, numattr = self.unpack_uint32(message, pos)
 
             attrs = {}
@@ -2506,7 +2517,7 @@ class FileSearchResult(PeerMessage):
                 for _ in range(numattr):
                     pos, attrnum = self.unpack_uint32(message, pos)
                     pos, attr = self.unpack_uint32(message, pos)
-                    attrs[str(attrnum)] = attr
+                    attrs[attrnum] = attr
 
             results.append((code, name.replace('/', '\\'), size, ext, attrs))
 
@@ -2678,13 +2689,14 @@ class FolderContentsResponse(PeerMessage):
             directory = directory.replace('/', '\\')
             pos, nfiles = self.unpack_uint32(message, pos)
 
+            ext = None
             shares[folder][directory] = []
 
             for _ in range(nfiles):
                 pos, code = self.unpack_uint8(message, pos)
                 pos, name = self.unpack_string(message, pos)
                 pos, size = self.unpack_uint64(message, pos)
-                pos, ext = self.unpack_string(message, pos)
+                pos, _ext = self.unpack_string(message, pos)  # Obsolete, ignore
                 pos, numattr = self.unpack_uint32(message, pos)
 
                 attrs = {}
@@ -2692,7 +2704,7 @@ class FolderContentsResponse(PeerMessage):
                 for _ in range(numattr):
                     pos, attrnum = self.unpack_uint32(message, pos)
                     pos, attr = self.unpack_uint32(message, pos)
-                    attrs[str(attrnum)] = attr
+                    attrs[attrnum] = attr
 
                 shares[folder][directory].append((code, name, size, ext, attrs))
 
